@@ -1,9 +1,5 @@
 
-/* Newfangled version identification scheme.
-
-   This scheme was added in Python 1.5.2b2; before that time, only PATCHLEVEL
-   was available.  To test for presence of the scheme, test for
-   defined(PY_MAJOR_VERSION).
+/* Python version identification scheme.
 
    When the major or minor version changes, the VERSION variable in
    configure.in must also be changed.
@@ -14,19 +10,24 @@
 /* Values for PY_RELEASE_LEVEL */
 #define PY_RELEASE_LEVEL_ALPHA	0xA
 #define PY_RELEASE_LEVEL_BETA	0xB
-#define PY_RELEASE_LEVEL_GAMMA	0xC
+#define PY_RELEASE_LEVEL_GAMMA	0xC     /* For release candidates */
 #define PY_RELEASE_LEVEL_FINAL	0xF	/* Serial should be 0 here */
 					/* Higher for patch releases */
 
 /* Version parsed out into numeric values */
-#define PY_MAJOR_VERSION	2
-#define PY_MINOR_VERSION	3
-#define PY_MICRO_VERSION	7
+/*--start constants--*/
+#define PY_MAJOR_VERSION	3
+#define PY_MINOR_VERSION	1
+#define PY_MICRO_VERSION	0
 #define PY_RELEASE_LEVEL	PY_RELEASE_LEVEL_FINAL
 #define PY_RELEASE_SERIAL	0
 
 /* Version as a string */
-#define PY_VERSION		"2.3.7"
+#define PY_VERSION      	"3.1"
+/*--end constants--*/
+
+/* Subversion Revision number of this file (not of the repository) */
+#define PY_PATCHLEVEL_REVISION  "$Revision: 73572 $"
 
 /* Version as a single 4-byte hex number, e.g. 0x010502B2 == 1.5.2b2.
    Use this for numeric comparisons, e.g. #if PY_VERSION_HEX >= ... */
