@@ -3,13 +3,11 @@
 Provides the Extension class, used to describe C/C++ extension
 modules in setup scripts."""
 
-# created 2000/05/30, Greg Ward
-
-__revision__ = "$Id: extension.py,v 1.9 2001/12/06 20:51:35 fdrake Exp $"
-
-import os, string
-from types import *
+__revision__ = "$Id: extension.py 73172 2009-06-03 11:17:15Z tarek.ziade $"
 
+import os
+import sys
+import warnings
 
 # This class is really only used by the "build_ext" command, so it might
 # make sense to put it in distutils.command.build_ext.  However, that
@@ -73,9 +71,22 @@ class Extension:
         used on all platforms, and not generally necessary for Python
         extensions, which typically export exactly one symbol: "init" +
         extension_name.
+      swig_opts : [string]
+        any extra options to pass to SWIG if a source file has the .i
+        extension.
+      depends : [string]
+        list of files that the extension depends on
+      language : string
+        extension language (i.e. "c", "c++", "objc"). Will be detected
+        from the source extensions if not provided.
+      optional : boolean
+        specifies that a build failure in the extension should not abort the
+        build process, but simply not install the failing extension.
     """
 
-    def __init__ (self, name, sources,
+    # When adding arguments to this constructor, be sure to update
+    # setup_keywords in core.py.
+    def __init__(self, name, sources,
                   include_dirs=None,
                   define_macros=None,
                   undef_macros=None,
@@ -86,11 +97,15 @@ class Extension:
                   extra_compile_args=None,
                   extra_link_args=None,
                   export_symbols=None,
+                  swig_opts = None,
+                  depends=None,
+                  language=None,
+                  optional=None,
+                  **kw                      # To catch unknown keywords
                  ):
-
-        assert type(name) is StringType, "'name' must be a string"
-        assert (type(sources) is ListType and
-                map(type, sources) == [StringType]*len(sources)), \
+        assert isinstance(name, str), "'name' must be a string"
+        assert (isinstance(sources, list) and
+                all(isinstance(v, str) for v in sources)), \
                 "'sources' must be a list of strings"
 
         self.name = name
@@ -105,13 +120,23 @@ class Extension:
         self.extra_compile_args = extra_compile_args or []
         self.extra_link_args = extra_link_args or []
         self.export_symbols = export_symbols or []
+        self.swig_opts = swig_opts or []
+        self.depends = depends or []
+        self.language = language
+        self.optional = optional
 
-# class Extension
+        # If there are unknown keyword options, warn about them
+        if len(kw) > 0:
+            options = [repr(option) for option in kw]
+            options = ', '.join(sorted(options))
+            msg = "Unknown Extension options: %s" % options
+            warnings.warn(msg)
 
+def read_setup_file(filename):
+    """Reads a Setup file and returns Extension instances."""
+    from distutils.sysconfig import (parse_makefile, expand_makefile_vars,
+                                     _variable_rx)
 
-def read_setup_file (filename):
-    from distutils.sysconfig import \
-         parse_makefile, expand_makefile_vars, _variable_rx
     from distutils.text_file import TextFile
     from distutils.util import split_quoted
 
@@ -125,7 +150,7 @@ def read_setup_file (filename):
                     lstrip_ws=1, rstrip_ws=1)
     extensions = []
 
-    while 1:
+    while True:
         line = file.readline()
         if line is None:                # eof
             break
@@ -136,10 +161,8 @@ def read_setup_file (filename):
             file.warn("'%s' lines not handled yet" % line)
             continue
 
-        #print "original line: " + line
         line = expand_makefile_vars(line, vars)
         words = split_quoted(line)
-        #print "expanded line: " + line
 
         # NB. this parses a slightly different syntax than the old
         # makesetup script: here, there must be exactly one extension per
@@ -160,7 +183,7 @@ def read_setup_file (filename):
             suffix = os.path.splitext(word)[1]
             switch = word[0:2] ; value = word[2:]
 
-            if suffix in (".c", ".cc", ".cpp", ".cxx", ".c++"):
+            if suffix in (".c", ".cc", ".cpp", ".cxx", ".c++", ".m", ".mm"):
                 # hmm, should we do something about C vs. C++ sources?
                 # or leave it up to the CCompiler implementation to
                 # worry about?
@@ -168,7 +191,7 @@ def read_setup_file (filename):
             elif switch == "-I":
                 ext.include_dirs.append(value)
             elif switch == "-D":
-                equals = string.find(value, "=")
+                equals = value.find("=")
                 if equals == -1:        # bare "-DFOO" -- no value
                     ext.define_macros.append((value, None))
                 else:                   # "-DFOO=blah"
@@ -188,11 +211,13 @@ def read_setup_file (filename):
                 append_next_word = ext.runtime_library_dirs
             elif word == "-Xlinker":
                 append_next_word = ext.extra_link_args
+            elif word == "-Xcompiler":
+                append_next_word = ext.extra_compile_args
             elif switch == "-u":
                 ext.extra_link_args.append(word)
                 if not value:
                     append_next_word = ext.extra_link_args
-            elif suffix in (".a", ".so", ".sl", ".o"):
+            elif suffix in (".a", ".so", ".sl", ".o", ".dylib"):
                 # NB. a really faithful emulation of makesetup would
                 # append a .o file to extra_objects only if it
                 # had a slash in it; otherwise, it would s/.o/.c/
@@ -203,15 +228,4 @@ def read_setup_file (filename):
 
         extensions.append(ext)
 
-        #print "module:", module
-        #print "source files:", source_files
-        #print "cpp args:", cpp_args
-        #print "lib args:", library_args
-
-        #extensions[module] = { 'sources': source_files,
-        #                       'cpp_args': cpp_args,
-        #                       'lib_args': library_args }
-
     return extensions
-
-# read_setup_file ()
