@@ -1,13 +1,13 @@
 
-import os, filecmp, shutil, tempfile
+import os, filecmp, shutil, tempfile, shutil
 import unittest
-from test import test_support
+from test import support
 
 class FileCompareTestCase(unittest.TestCase):
     def setUp(self):
-        self.name = test_support.TESTFN
-        self.name_same = test_support.TESTFN + '-same'
-        self.name_diff = test_support.TESTFN + '-diff'
+        self.name = support.TESTFN
+        self.name_same = support.TESTFN + '-same'
+        self.name_diff = support.TESTFN + '-diff'
         data = 'Contents of file go here.\n'
         for name in [self.name, self.name_same, self.name_diff]:
             output = open(name, 'w')
@@ -46,10 +46,16 @@ class DirCompareTestCase(unittest.TestCase):
         self.dir = os.path.join(tmpdir, 'dir')
         self.dir_same = os.path.join(tmpdir, 'dir-same')
         self.dir_diff = os.path.join(tmpdir, 'dir-diff')
+        self.caseinsensitive = os.path.normcase('A') == os.path.normcase('a')
         data = 'Contents of file go here.\n'
         for dir in [self.dir, self.dir_same, self.dir_diff]:
+            shutil.rmtree(dir, True)
             os.mkdir(dir)
-            output = open(os.path.join(dir, 'file'), 'w')
+            if self.caseinsensitive and dir is self.dir_same:
+                fn = 'FiLe'     # Verify case-insensitive comparison
+            else:
+                fn = 'file'
+            output = open(os.path.join(dir, fn), 'w')
             output.write(data)
             output.close()
 
@@ -93,33 +99,36 @@ class DirCompareTestCase(unittest.TestCase):
     def test_dircmp(self):
         # Check attributes for comparison of two identical directories
         d = filecmp.dircmp(self.dir, self.dir_same)
-        self.failUnless(d.left_list == d.right_list == ['file'])
-        self.failUnless(d.common == ['file'])
+        if self.caseinsensitive:
+            self.assertEqual([d.left_list, d.right_list],[['file'], ['FiLe']])
+        else:
+            self.assertEqual([d.left_list, d.right_list],[['file'], ['file']])
+        self.assertEqual(d.common, ['file'])
         self.failUnless(d.left_only == d.right_only == [])
-        self.failUnless(d.same_files == ['file'])
-        self.failUnless(d.diff_files == [])
+        self.assertEqual(d.same_files, ['file'])
+        self.assertEqual(d.diff_files, [])
 
         # Check attributes for comparison of two different directories
         d = filecmp.dircmp(self.dir, self.dir_diff)
-        self.failUnless(d.left_list == ['file'])
+        self.assertEqual(d.left_list, ['file'])
         self.failUnless(d.right_list == ['file', 'file2'])
-        self.failUnless(d.common == ['file'])
-        self.failUnless(d.left_only == [])
-        self.failUnless(d.right_only == ['file2'])
-        self.failUnless(d.same_files == ['file'])
-        self.failUnless(d.diff_files == [])
+        self.assertEqual(d.common, ['file'])
+        self.assertEqual(d.left_only, [])
+        self.assertEqual(d.right_only, ['file2'])
+        self.assertEqual(d.same_files, ['file'])
+        self.assertEqual(d.diff_files, [])
 
         # Add different file2
         output = open(os.path.join(self.dir, 'file2'), 'w')
         output.write('Different contents.\n')
         output.close()
         d = filecmp.dircmp(self.dir, self.dir_diff)
-        self.failUnless(d.same_files == ['file'])
-        self.failUnless(d.diff_files == ['file2'])
+        self.assertEqual(d.same_files, ['file'])
+        self.assertEqual(d.diff_files, ['file2'])
 
 
 def test_main():
-    test_support.run_unittest(FileCompareTestCase, DirCompareTestCase)
+    support.run_unittest(FileCompareTestCase, DirCompareTestCase)
 
 if __name__ == "__main__":
     test_main()
