@@ -683,8 +683,13 @@ class HandlerTests(unittest.TestCase):
                 self.msg = msg
                 self.status = status
                 self.reason = reason
+                self.code = 200
             def read(self):
                 return ''
+            def info(self):
+                return {}
+            def geturl(self):
+                return self.url
         class MockHTTPClass:
             def __init__(self):
                 self.level = 0
@@ -1045,7 +1050,7 @@ class HandlerTests(unittest.TestCase):
         self.assertFalse(http_handler.requests[0].has_header(auth_header))
         userpass = bytes('%s:%s' % (user, password), "ascii")
         auth_hdr_value = ('Basic ' +
-            base64.encodestring(userpass).strip().decode())
+            base64.encodebytes(userpass).strip().decode())
         self.assertEqual(http_handler.requests[1].get_header(auth_header),
                          auth_hdr_value)
 
